import os

import subprocess

from . import logs
from . import __version__


# __version__ = '0.0.2'


@logs.log_function
def expand_path(path):
    user_expanded = os.path.expanduser(path)
    vars_expanded = os.path.expandvars(user_expanded)
    abs_path = os.path.abspath(vars_expanded)
    return abs_path


@logs.log_generator
def dir_combinations():
    for name in ('Music', 'Musica', 'Download'):
        lower = name.lower()
        yield lower
        yield lower + 's'
        upper = name.upper()
        yield upper
        yield upper + 'S'
        if len(name) > 1:
            yield upper[0] + lower[1:]
            yield upper[0] + lower[1:] + 's'


@logs.log_function
def default_music_path():
    base = dir_combinations()
    base = (f'~/{n}/' for n in base)
    path_list = []
    for path in base:
        user_expanded = os.path.expanduser(path)
        vars_expanded = os.path.expandvars(user_expanded)
        abs_path = os.path.abspath(vars_expanded)
        if os.path.isdir(abs_path):
            logs.logging.info(f'found music dir as {abs_path}')
            path_list.append(abs_path)
        logs.logging.info(f'{abs_path} is not a music dir')
    return path_list


@logs.log_function
def default_config_path():
    ''' Get user's configuration directory.'''
    if os.name == 'nt':
        # windows system
        confdir = os.environ['APPDATA']

    elif 'XDG_CONFIG_HOME' in os.environ:
        confdir = os.environ['XDG_CONFIG_HOME']
    else:
        confdir = os.path.join(os.path.expanduser('~'), '.config')

    config_path = os.path.join(confdir, 'py-playlist')
    return config_path


@logs.log_function
def normalize_config_path(config_path = None):
    '''
    normalize and makes the configuration directory
    
    If `config_path` is not informed, the default user configuration path is
    used
    '''
    if config_path is None or config_path == '':
        config_path = default_config_path()
        os.makedirs(config_path, exist_ok=True)
    elif not os.path.isfile(config_path):
        os.makedirs(config_path, exist_ok=True)

    return config_path


@logs.log_function
def call(command, *args):
    if ' ' in command:
        logs.logging.critical(f'the comand "{command}" contains a space in it')
        raise Exception(f'unsafe call to command "{command}"')
    return subprocess.call([command, *args])



import click
import click_aliases
import logging
import os


from . import utils
from . import config
from . import playlist
from .logs import log_function, start_logs


@log_function
def validate_dir_list(path_list):
    validated = []
    for path in path_list:
        expanded = utils.expand_path(path)
        if os.path.isdir(expanded):
            validated.append(expanded)
        else:
            logging.error(f'dir {path} is not valid')
    return validated


@click.group(invoke_without_command=True, cls=click_aliases.ClickAliasedGroup)
@click.option('--config_path', '--config', '-c', default=None, help='''set a configuration directory to be considered''')
@click.option('--playlist_path', '--playlist', '-l', default=None, help='''set the playplist dir for one run''')
@click.option('--editor', '-e', default=None, help='''set the editor for one run''')
@click.option('--editor_args', '--eargs', default=None, help='''set the editor arguments, to parametrize its call''')
@click.option('--player', '-p', default=None, help='''set the player for one run''')
@click.option('--player_args', '--pargs', default=None, help='''set the player arguments, to parametrize its call''')
@click.option('--music_path', '--music', '-m', 'music_path_list', multiple=True, default=None, help='''set the
        music dir for one run

        This option can be called multiple times and it will erase previous
        music paths''')
@click.option('--add_music_path', '--add_music', '-a', 'music_path_add', multiple=True, default=None, help='''adds the
        music dir for one run

        This option can be called multiple times and it will not erase previous
        music paths''')
@click.option('--save_config/', '--save/', '-s/-S', help='''updates the config with the parameters provided in this call''')
@click.option('--force_default/', '--default/', '-d/-D', help='''loads the
        default configuration''')
@click.option('--debug', 'log_level', flag_value=logging.DEBUG, help='''set the log level to debug''')
@click.option('--info', 'log_level', flag_value=logging.INFO, help='''set the log level to info''')
@click.option('--warning', 'log_level', flag_value=logging.WARNING, help='''set the log level to warning''')
@click.option('--error', 'log_level', flag_value=logging.ERROR, help='''set the log level to error''')
@click.version_option(utils.__version__)
def main(config_path, save_config,
        force_default, log_level=logging.CRITICAL, music_path_list=[],
        music_path_add=[], **kwargs):
    '''
    Entry point to py_playlist

    This can be called without a command to execute configurations, and can
    receive some parameters to change the behavior, calling different players
    and file editors.
    '''
    start_logs(log_level)

    cleaned_music_path = validate_dir_list(music_path_list)
    cleaned_path_add = validate_dir_list(music_path_add)

    if len(music_path_list) > 0 and len(cleaned_music_path) == 0 and len(cleaned_path_add) == 0:
        logging.critical(f'no music dir is valid')
        raise Exception('No valid music path')

    config.config_init(config_path, save_config, force_default, cleaned_music_path, cleaned_path_add, **kwargs)


@main.command(aliases=['ed', 'e'])
@click.argument('playlist_name')
def edit(playlist_name):
    '''
    open a file editor to create and edit a playlist
    '''
    playlist.edit(playlist_name)


@main.command(aliases=['pl', 'p'])
@click.argument('playlist_name')
@click.option('--shuffle', '-s', flag_value=True, help='''allows shuffling the songs''')
def play(playlist_name, shuffle=False):
    '''
    runs the playlist in the configured player
    '''
    playlist.play(playlist_name, shuffle)


@main.command(aliases=['ls', 'l'])
def list():
    '''
    show the playlists list
    '''
    playlist.list_()


@main.command(aliases=['ec', 'c'])
@click.option('--config_path', '--config', '-c', default=None, help='''set a configuration directory to be considered''')
def edit_conf(config_path):
    '''
    open a file editor to create and edit the configuration
    '''
    config.edit_config(config_path)


@main.command(aliases=['sc', 's'])
@click.option('--config_path', '--config', '-c', default=None, help='''set a configuration directory to be considered''')
def show(config_path):
    '''
    show the configuration
    '''
    config.show_config(config_path)


if __name__ == '__main__':
    main(obj={})



import typing as tp
import os
import subprocess
import json
import pprint

from . import utils
from . import logs

_config_path = None
_config = None

class Configurations(tp.NamedTuple):
    playlist_path: str = utils.normalize_config_path(None)
    music_path_list: tp.Optional[tp.List[str]] = utils.default_music_path()
    editor: str = os.environ.get('EDITOR', 'vim')
    player: str = 'mpv'
    player_args: tp.List[str] = []
    editor_args: tp.List[str] = []
    version: str = utils.__version__


def assert_config(config):
    '''
    asserts configuration values tu avoid shell injection attacks
    '''
    if ' ' in config.editor:
        raise Exception('Space char in editor configuration, pleas use the `editor_args` to parametrize the editor')

    if ' ' in config.player:
        raise Exception('Space char in player configuration, pleas use the `player_args` to parametrize the player') 


@logs.log_function
def main_conf_path(config_path):
    if os.path.isdir(config_path):
        return os.path.join(config_path, 'main_conf.json')
    return config_path


@logs.log_function
def default_config(config_path, **kwargs):
    '''
    default_config(config_path, **kwargs)

    Build up new configuration objetct, may receive any configuration field in
    the kwargs.

    The `config_path` must be informed to set a default `playlist_path` in the
    configuration, as it has no default value by itself.
    '''
    kwargs.setdefault('playlist_path', config_path)
    parameters = {key: value for key, value in kwargs.items() if value}
    config = Configurations(**parameters)
    return config


@logs.log_function
def load_config(config_path):
    with open(main_conf_path(config_path), 'r') as file_buf:
        obj = json.load(file_buf)
        return Configurations(**obj)


@logs.log_function
def write_config(config, config_path):
    obj = config._asdict()
    with open(main_conf_path(config_path), 'w') as file_buf:
        json.dump(obj, file_buf, indent=4, sort_keys=True)

@logs.log_function
def update_config(config, music_path_list, music_path_add, **kwargs):
    obj = config._asdict()

    if music_path_list:
        obj['music_path_list'] = music_path_list
    if music_path_add:
        obj.setdefault('music_path_list', [])
        obj['music_path_list'] = tuple(( *music_path_add, *obj['music_path_list'] ))

    parameters = {key: value for key, value in kwargs.items() if value}
    obj.update(parameters)

    if isinstance(obj['player_args'], str):
        obj['player_args'] = obj['player_args'].split(' ')

    if isinstance(obj['editor_args'], str):
        obj['editor_args'] = obj['editor_args'].split(' ')

    return Configurations(**obj)

@logs.log_function
def config_init(config_path, save_config, force_default, music_path_list, music_path_add, **kwargs):

    config_path = utils.normalize_config_path(config_path)
    global _config_path
    _config_path = config_path

    logs.logging.info(f'normalized config path is {config_path}')

    main_conf_file = main_conf_path(config_path)

    logs.logging.info(f'main config file is {main_conf_file}')

    if force_default:
        logs.logging.debug(f'forced default configurations, re initializing with default values')
        config = default_config(config_path, **kwargs)
    elif not os.path.isfile(main_conf_file):
        logs.logging.debug(f'main configuration file do not exists, initializing with default values')
        config = default_config(config_path, **kwargs)
    else:
        logs.logging.debug(f'main configuration file exists, loading')
        config = load_config(config_path)
    logs.logging.info(f'loaded configuration {str(config)}')

    config = update_config(config, music_path_list, music_path_add, **kwargs)
    logs.logging.info(f'updated configuration to {str(config)}')

    if save_config:
        logs.logging.debug('saving configurations into path {config_path}')
        write_config(config, config_path)

    assert_config(config)

    global _config
    _config = config


@logs.log_function
def edit_config(config_path):
    dir_path = _config_path
    if config_path:
        dir_path = utils.normalize_config_path(config_path)
    
    path = main_conf_path(dir_path)

    if not os.path.isfile(path):
        logs.logging.critical(f'file do not exists')
        return
    
    utils.call(_config.editor, *_config.editor_args, path)


@logs.log_function
def show_config(config_path):
    config = None
    if config_path:
        dir_path = utils.normalize_config_path(config_path)
        path = main_conf_path(dir_path)
        if not os.path.isfile(path):
            logs.logging.critical(f'file do not exists')
            return
        config = load_config(path)
    else:
        config = _config
    
    pprint.pprint({**config._asdict()})



import logging
import functools


def start_logs(log_level):
    logging.basicConfig(level=log_level)


def log_function(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        args_str = tuple(repr(a) for a in args)
        kwargs_str = tuple(f'{k}={repr(v)}' for k,v in kwargs.items())
        sig = ', '.join(args_str + kwargs_str)
        logging.debug(f'calling {func.__name__}({sig})')
        return_value = func(*args, **kwargs)
        logging.debug(f'{func.__name__} returning value {repr(return_value)}')
        return return_value
    return wrapper


def log_generator(gen_func):
    @functools.wraps(gen_func)
    def wrapper(*args, **kwargs):
        args_str = tuple(repr(a) for a in args)
        kwargs_str = tuple(f'{k}={repr(v)}' for k,v in kwargs.items())
        sig = ', '.join(args_str + kwargs_str)
        logging.debug(f'calling {gen_func.__name__}({sig})')
        gen = gen_func(*args, **kwargs)
        for value in gen:
            logging.debug(f'yielding from {gen_func.__name__} value {repr(value)}')
            yield value
    return wrapper



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


# print(setuptools.find_packages('py_playlist'))
# print(setuptools.find_packages('src'))
# raise Exception()

setuptools.setup(
    name="py_playlist",
    version="0.0.3",
    author="Gustavo Henrique Spiess",
    author_email="gustavospiess@gmail.com",
    description="A playlist manager for terminal based players",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/gustavospiess/py-playlist",
    package_dir={'':'src'},
    # packages=setuptools.find_packages('py_playlist'),
    packages=['py_playlist'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    package_data={"": ["LICENSE", "README.rst", "CHANGELOG"]},
    install_requires=['click>=7.1.2', 'click-aliases>=1.0.1'],
    entry_points={
        'console_scripts': [
            'py_playlist=py_playlist.__main__:main',
            'ppl=py_playlist.__main__:main',
        ],
    },
)



import os
import random
import pprint

from . import config
from . import logs
from . import utils

@logs.log_function
def playlist_path(playlist_name):
    return os.path.join(config._config.playlist_path, playlist_name)


@logs.log_function
def edit(playlist_name):
    '''
    open a file editor to create and edit a playlist
    '''
    if len(config._config.music_path_list):
        os.chdir(config._config.music_path_list[0])
    utils.call(config._config.editor, config._config.editor_args, playlist_path(playlist_name))


@logs.log_generator
def normalized_song_gen(song_iter):
    for song_path in song_iter:
        logs.logging.debug(f'normalizing {repr(song_path)}')
        trimmed = song_path.strip()
        if not trimmed:
            logs.logging.info('empty line in playlist')
            continue
        abs_path = utils.expand_path(trimmed)
        if os.path.isfile(abs_path):
            yield abs_path
            continue

        logs.logging.info(f'song {song_path} not found as absolute path')

        for music_dir in config._config.music_path_list:
            path_in_dir = os.path.join(music_dir, trimmed)
            abs_path = utils.expand_path(path_in_dir)
            if os.path.isfile(abs_path):
                yield abs_path
            logs.logging.info(f'song {song_path} not found in {music_dir}')



@logs.log_function
def play(playlist_name, shuffle):
    '''
    runs the playlist in the configured player
    '''
    with open(playlist_path(playlist_name), 'r') as file_buffer:
        norm = list(normalized_song_gen(file_buffer))
        logs.logging.debug(f'normalized playlist is {repr(norm)}')
        if shuffle:
            logs.logging.debug(f'shuffle songs')
            random.shuffle(norm)

        utils.call(config._config.player, config._config.player_args, *norm)


def list_():
    pprint.pprint([*os.listdir(config._config.playlist_path)])
